//
//  Sight.m
//  TouristHelper
//
//  Adam Eberbach

#import "Sight.h"
#import "AFImageRequestOperation.h"

@interface Sight ()
@property (nonatomic, strong) NSString *baseURLString;
@property (nonatomic, strong) UIImage *thumbnail;
@end

@implementation Sight

- (id)initWithTitle:(NSString *)title
         coordinate:(CLLocationCoordinate2D)coordinate
            baseURL:(NSString *)baseURLString {
  
  self = [super init];
  if(self) {
    self.title = title;
    self.coordinate = coordinate;
    self.baseURLString = baseURLString;
  }
  return self;
}

// initiate image fetches
- (UIImage *)thumbnailImage {
  
  if(_thumbnail == nil) {
    NSString *urlString = [NSString stringWithFormat:@"%@_s.jpg", _baseURLString];
    AFImageRequestOperation* operation
    = [AFImageRequestOperation imageRequestOperationWithRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:urlString]] imageProcessingBlock:nil
                                                        success:^(NSURLRequest *request, NSHTTPURLResponse *response, UIImage *image) {
                                                          self.thumbnail = image;
                                                          [_delegate sightDidUpdateAvailableThumbnail:self];
                                                        }
                                                        failure:nil];
    [operation start];
  }
  return _thumbnail;
}

- (void)initiateImageDisplay {
  if (_fullImage) {
    [_delegate sightDisplayImage:self];
  } else {
    [_delegate sightBeginningLargeImageDownload];
    NSString *urlString = [NSString stringWithFormat:@"%@_b.jpg", _baseURLString];
    AFImageRequestOperation* operation = [AFImageRequestOperation
                                          imageRequestOperationWithRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:urlString]]
                                          imageProcessingBlock:nil success:^(NSURLRequest *request, NSHTTPURLResponse *response, UIImage *image) {
                                            [_delegate sightsDownloadEnded];
                                            self.fullImage = image;
                                            [_delegate sightDisplayImage:self];
                                          } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error) {
                                            [_delegate sightsDownloadEnded];
                                          }];
    [operation start];
    return;
  }
}

- (NSString *)description {
  
  return [NSString stringWithFormat:@"Sight titled '%@' at location lat:%f long:%f base image URL %@",
          _title,
          _coordinate.latitude,
          _coordinate.longitude,
          _baseURLString];
}
@end
